(* ::Package:: *)

BeginPackage["ATP`"]

Print["This is APT.m, version 1.1, 
a tool for computing \!\(\*SubsuperscriptBox[\(\[Del]\), \(T\), \(s\)]\), a generalisation of Alexander polynomials for tangles,
written in Mathematica 10.3.1.0 for Linux x86 (64-bit). 
A manual for this Mathematica package can be found in appendix C of

   C.B. Zibrowius, ",Style["On a Heegaard Floer theory for tangles", Italic],", PhD thesis (2016), Cambridge.

For questions, suggestions or comments, please email claudius.zibrowius","@","gmail.com. 

    Copyright (C) 2015-2016 Claudius Zibrowius

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>."
];

(* This is a Mathematica package for computing Alexander polynomials for tangles. For more information and details, see the manual. This is version 1.1. 'APT' stands for 'Alexander Polynomial for Tangles'. *)

(* The variable v is the input data for a tangle, taking the form of a matrix with one row for each crossing/alpha-curve and 7 columns;
However: 1st row = boundary regions;
For all following rows,
1st column: between over and under,
2nd-4th col: turn anti-clockwise,
5th column: follow over-strand, the under-strand points to the left/right (L/R),
6th column: colour over-strand,
7th column: colour under-strand;

Note: Do not use h as a name for a boundary region in a 4-ended tangle, since h is reserved for a homological grading. For internal regions, this is no problem;
*)

(* Alexander Codes for positive(=L) and negative(=R) crossings *)
L[region_,o_,u_]:={\[Delta]^(1/2) o u,o u^-1,\[Delta]^(1/2) h^-1 o^-1 u^-1,o^-1 u}[[region]];
R[region_,o_,u_]:={\[Delta]^(-1/2) o^-1 u^-1,o u^-1,\[Delta]^(-1/2) h o u,o^-1 u}[[region]];

(* Preparing input data for algorithm *)
Prepare1[v_]:=Map[Drop[#,-3]&,Drop[v,1]]; (* cut out the region info for each crossing *)
Prepare2[v_]:=Transpose/@(v/.Array[v[[#]]->{v[[#]],{1,2,3,4}}&,Length[v]]); (* add the region numbers to region infos and transpose *)
Prepare3[v_]:=Map[Drop[#,4]&,Drop[v,1]]; (* cut out L/R and over/under info for each crossing *)

(* basic idea for calculating generators:
1) Tt: iteratively determine all possibilities for first n entries.
2) BoundaryCond: delete those for which #(occupied beta-arcs) is incorrect.
  3) T: Put 1) and 2) together.                                               *)
Tt[v_]:=Fold[Function[{valu,n},DeleteCases[Flatten[Outer[Append,valu,v[[n,1;;4]],1],1],list_/;list[[All,1]]=!=DeleteDuplicates[list[[All,1]]]]],{{v[[1,1]]},{v[[1,2]]},{v[[1,3]]},{v[[1,4]]}},Array[#+1&,Length[v]-1]];
BoundaryCond[boundary_,lengthv_,potentialT_]:=DeleteCases[potentialT,list_/;(lengthv-Length[DeleteCases[list,bob_/;MemberQ[boundary,bob[[1]]]]]-(Length[boundary]/2)+1)=!=0];
T[v_]:=BoundaryCond[v[[1]],Length[v]-1,Tt[Prepare2[Prepare1[v]]]];

(* various lists of "properties" of each generator in T[v] *)
(* Alexander coefficients of each region at each crossing *)
LocalCoefftable[v_]:=Array[v[[#1]][[1]][#2,v[[#1]][[2]],v[[#1]][[3]]]&,{Length[v],4}];
(* table of Alexander coefficients of each generator *)
Coefftable[T_,Lct_]:=Array[Product[Lct[[iii,T[[#,iii,2]]]],{iii,1,Length[T[[1]]]}]&,Length[T]];
(* generator names=concatenation of regions *)
GenNametable[T_]:=Array[ToExpression[StringReplace[ToString[T[[#,All,1]]],{"{"-> "",", "->"","}"->""}]]&,Length[T]];
(* names of sites (concatenated) (optional, not used below) *)
BoundaryNamestable[T_,boundary_]:=Array[ToExpression[StringReplace[ToString[Sort[DeleteCases[T[[#,All,1]],bob_/;MemberQ[boundary,bob]=!=True]]],{"{"-> "",", "->"","}"->""}]]&,Length[T]]
(* name of sites (in product form/not concatenated) Note: Appending 1 is for the long knot/link case *)
BoundaryProducttable[T_,boundary_]:=Array[ToExpression[StringReplace[ToString[Sort[Append[DeleteCases[T[[#,All,1]],bob_/;MemberQ[boundary,bob]=!=True],1]]],{"{"-> "",", "->"*","}"->""}]]&,Length[T]]
(* name of sites (in product form/not concatenated) (alternative version, but unfortunately not faster) *)
BoundaryProductAlttable[T_,boundary_]:=Array[Product[If[MemberQ[boundary,T[[#,iii,1]]]=!=True,1,T[[#,iii,1]]],{iii,Length[T[[1,All,1]]]}]&,Length[T]]

(* Alexander polynomials *)
AlexpolyPdh[v_]:=Coefftable[T[v],LocalCoefftable[Prepare3[v]]].BoundaryProducttable[T[v],v[[1]]];
AlexpolyPd[v_]:=AlexpolyPdh[v]/.h->1;
AlexpolyPh[v_]:=AlexpolyPdh[v]/.\[Delta]->1;
AlexpolyP[v_]:=AlexpolyPh[v]/.h->-1;
AlexpolyGdh[v_]:=Coefftable[T[v],LocalCoefftable[Prepare3[v]]].(GenNametable[T[v]]*BoundaryProducttable[T[v],v[[1]]]);
AlexpolyGd[v_]:=AlexpolyGdh[v]/.h->1;
AlexpolyGh[v_]:=AlexpolyGdh[v]/.\[Delta]->1;
AlexpolyG[v_]:=AlexpolyGd[v]/.h->-1;

(* Diagrammatic Outputs *)
(* basic operation polynomial\[Rule]grid *)
Pexp[poly_]:=If[poly===0,0,Max[Exponent[poly,p^-1],0]]
PExp[poly_]:=If[poly===0,0,Max[Exponent[poly,p],0]]
Qexp[poly_]:=If[poly===0,0,Max[Exponent[poly,q^-1],0]]
QExp[poly_]:=If[poly===0,0,Max[Exponent[poly,q],0]]
Deltaexp[poly_]:=If[poly===0,0,Max[Exponent[poly,\[Delta]^-1],0]]
DeltaExp[poly_]:=If[poly===0,0,Max[Exponent[poly,\[Delta]],0]]
SimpleGrid[poly_]:=If[poly===0,Grid[{{0}},Frame->All,FrameStyle->Directive[Thickness[0.1],AbsoluteDashing[{100,0.01}],Gray], Background->{Gray}],Grid[CoefficientList[(poly+zeroz)*p^Pexp[poly]*q^Qexp[poly],{p,q}]/.zeroz->0,Frame->All,FrameStyle->Directive[Thickness[0.1],AbsoluteDashing[{100,0.01}],Gray],Background->{{Qexp[poly]+1->LightGray},{Pexp[poly]+1->LightGray},{Pexp[poly]+1,Qexp[poly]+1}->Gray }]]/.{0->ToExpression[ " "]}

(* Final Output (filtering and ordering of coefficients according to their beta-arc-indexing set)*)
(* beta-arcs sorting *)
Barcs[barc_]:=Union[Sort/@DeleteCases[Tuples[barc,Length[barc]/2-1],list_/;list=!=DeleteDuplicates[list]]];
(* Concatenation of generator tuple entries *)
Conc:=Function[{v},ToExpression[StringReplace[ToString[v],{"{"-> "",", "->"","}"->""}]]]

(* Gray shades table output *)
Eval[poly_,barc_,iii_]:=If [Length[Barcs[barc]]<2,SimpleGrid[poly],{Product[Barcs[barc][[iii]][[jjj]],{jjj,Length[barc]/2-1}],SimpleGrid[Coefficient[poly/.{Product[Barcs[barc][[iii]][[jjj]],{jjj,Length[barc]/2-1}]->x},x]]}];
Addx[exp_]:=ToExpression[StringJoin[ToString[exp],"x"]]; (* one-crossing case, this is to distinguish the generator names from the coefficients *)
AlexpolytableGdh[v_,poly_]:=MatrixForm[Table[Eval[poly,v[[1]],iii],{iii,Length[Barcs[v[[1]]]]}]];
AlexpolytableGd[v_,poly_]:=AlexpolytableGdh[v,poly/.h->1];
AlexpolytableGh[v_,poly_]:=AlexpolytableGdh[v,poly/.\[Delta]->1];
AlexpolytablePdh[v_,poly_]:=If[Length[v]==2, AlexpolytableGdh[v,poly/.Thread[Map[Addx,Map[Conc,T[v]]]->1]],AlexpolytableGdh[v,poly/.Thread[Map[Conc,T[v][[All,All,1]]]->1]]]; (* forgetting about generator specification *)
AlexpolytablePd[v_,poly_]:=AlexpolytablePdh[v,poly/.h->1];
AlexpolytablePh[v_,poly_]:=AlexpolytablePdh[v,poly/.\[Delta]->1];
AlexpolytableP[v_,poly_]:=AlexpolytablePh[v,poly/.h->-1]; 

(* displaying negative powers as exponents,not as fractions *)
Unprotect[Power];
Power/:MakeBoxes[Power[$z_,Rational[$b_?NonNegative,$a_]],fmt_]:=RowBox[{SuperscriptBox[ToBoxes[$z],FractionBox[$b,$a]]}];
Power/:MakeBoxes[Power[$z_,Rational[$b_?Negative,$a_]],fmt_]:=RowBox[{SuperscriptBox[ToBoxes[$z],RowBox[{"-",FractionBox[-$b,$a]}]]}];
Power/:MakeBoxes[Power[$z_,$b_?Negative],fmt_]:=RowBox[{SuperscriptBox[ToBoxes[$z],$b]}];
Protect[Power];
Unprotect[Times];
Times/:MakeBoxes[Times[$c_,Power[$z_,Rational[$b_?NonNegative,$a_]]],fmt_]:=RowBox[{MakeBoxes[$c,fmt],SuperscriptBox[ToBoxes[$z],FractionBox[$b,$a]]}];
Times/:MakeBoxes[Times[$c_,Power[$z_,Rational[$b_?Negative,$a_]]],fmt_]:=RowBox[{MakeBoxes[$c,fmt],SuperscriptBox[ToBoxes[$z],RowBox[{"-",FractionBox[-$b,$a]}]]}];
Times/:MakeBoxes[Times[$c_,Power[$z_,$b_?Negative]],fmt_]:=RowBox[{MakeBoxes[$c,fmt],SuperscriptBox[ToBoxes[$z],$b]}];
Protect[Times];
(* calculate boundary configurations *)
(*region labels*)
alphabet=DeleteCases[CharacterRange["a","z"],v_/;v==="p"||v==="q"];
NextLetter[tangle_]:=ToExpression[If[#==={},Print["Your tangle diagrams are really large, I have run out of letters. Redefine NextLetter[tangle_] to fix this."];a1,#[[1]]]&@Complement[alphabet,ToString/@Sort[DeleteDuplicates[DeleteCases[Flatten[tangle],v_/;v===R||v===L||v===p||v===q]]]]];
(* detecting orientations and colours of ends from input *)
StrandNeighbouringRegions[end_,tangle_]:=Prepend[Table[#[[i;;i+1]],{i,1,Length[#]-1}]&@tangle[[1]],{tangle[[1,-1]],tangle[[1,1]]}][[end]];
(* find a crossing adjacent to both of the two neighbouring regions of the end; Careful: there might be more than one such crossing. *)
(*  
If[Length[#]\[Equal]1,#[[1]],Print["Error: StrandCrossing["<>ToString[end]<>","<>ToString[tangle]<>"] does not find a single crossing: The output is:\n"<>ToString[#]]]&@
*)
StrandCrossing[end_,tangle_]:=FirstCase[Drop[tangle,1],v_/;AllTrue[StrandNeighbouringRegions[end,tangle],MemberQ[v,#]&]];
(* calculate the quotient of the Alexander gradings of the two neighbouring regions of the end at a crossing adjacent to both of these regions; there might be more than one such crossing, but the colour is always the same. *)
StrandCoeff[end_,tangle_]:=#[[5]][Position[#,StrandNeighbouringRegions[end,tangle][[1]]][[1,1]],#[[6]],#[[7]]]/#[[5]][Position[#,StrandNeighbouringRegions[end,tangle][[2]]][[1,1]],#[[6]],#[[7]]]&@StrandCrossing[end,tangle];
(* calculate colour of the strand from the StrandCoeff *)
StrandColour[end_,tangle_]:=Variables[StrandCoeff[end,tangle]/.{\[Delta]->1,h->1}][[1]];
AllStrandColors[tangle_]:=DeleteDuplicates[Flatten[Drop[tangle,1][[All,{6,7}]]]];
(* calculate position of the two neighbouring regions in 'StrandCrossing' *)
StrandPos[end_,tangle_]:=Flatten[Join[Position[#,StrandNeighbouringRegions[end,tangle][[1]]],Position[#,StrandNeighbouringRegions[end,tangle][[2]]]]]&@StrandCrossing[end,tangle]
(* Usually, in each case, only the first condition in (#1 || #2) should apply. The second is for the case that there are multiple crossings that StrandCrossing might find *)
StrandOrient[end_,tangle_]:=\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{"\"\<in\>\"", 
RowBox[{
RowBox[{"MemberQ", "[", 
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"{", 
RowBox[{"2", ",", "3"}], "}"}], ",", 
RowBox[{"{", 
RowBox[{"3", ",", "4"}], "}"}]}], "}"}], ",", 
RowBox[{"StrandPos", "[", 
RowBox[{"end", ",", "tangle"}], "]"}]}], "]"}], "||", 
RowBox[{"MemberQ", "[", 
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"{", 
RowBox[{"1", ",", "4"}], "}"}], ",", 
RowBox[{"{", 
RowBox[{"2", ",", "1"}], "}"}]}], "}"}], ",", 
RowBox[{"StrandPos", "[", 
RowBox[{"end", ",", "tangle"}], "]"}]}], "]"}]}]},
{"\"\<out\>\"", 
RowBox[{
RowBox[{"MemberQ", "[", 
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"{", 
RowBox[{"4", ",", "1"}], "}"}], ",", 
RowBox[{"{", 
RowBox[{"1", ",", "2"}], "}"}]}], "}"}], ",", 
RowBox[{"StrandPos", "[", 
RowBox[{"end", ",", "tangle"}], "]"}]}], "]"}], "||", 
RowBox[{"MemberQ", "[", 
RowBox[{
RowBox[{"{", 
RowBox[{
RowBox[{"{", 
RowBox[{"3", ",", "2"}], "}"}], ",", 
RowBox[{"{", 
RowBox[{"4", ",", "3"}], "}"}]}], "}"}], ",", 
RowBox[{"StrandPos", "[", 
RowBox[{"end", ",", "tangle"}], "]"}]}], "]"}]}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
(* Graphical representation of boundary configuration *)
Coords[i_,tangle_]:={Cos[Pi/Length[tangle[[1]]]+2Pi*i/Length[tangle[[1]]]],Sin[Pi/Length[tangle[[1]]]+2Pi*i/Length[tangle[[1]]]]};
DConfig::usage="DConfig[tangle_] graphically represents the boundary configuration of the tangle 'tangle', i.e. the open components, their orientations and colours, their index and the boundary regions.";
DConfig[tangle_]:=Graphics[Join[
{Circle[{0,0},1],Arrowheads[Medium]},
Table[
{If[StrandOrient[i,tangle]==="out",Arrow[{Coords[i,tangle],Coords[i,tangle]*1.8}],Arrow[{Coords[i,tangle]*1.8,Coords[i,tangle]}]],Text[StrandColour[i,tangle],Coords[i,tangle]*2],Text[i,Coords[i,tangle]*0.7],Text[tangle[[1,i]],Coords[i+0.5,tangle]*1.5]},
{i,1,Length[tangle[[1]]]}],
{Transparent,Circle[{0,0},1.3]}
],ImageSize->Tiny]
(* twisting general 2n-ended tangles *)
Twist::usage="Twist[tangle_,boundaryregion_,twist_] computes a new tangle obtained from 'tangle' by adding a single crossing at the region 'boundaryregion', where 'twist' determines if the new crossing is a positive (L) or negative (R) crossing.";
Twist[tangle_,boundaryregion_,twist_]:=Module[
{
INS1, (* lower index of neighbouring strand *)
INS2,
newrow,
colour1,
colour2,
r1=boundaryregion,(* those are the regions for LR=L and both arrows outward *)
r2,
r3,
r4
}
,
If[MemberQ[{L,R},twist]==False,Print["Input Error: The last argument ('twist') should be either 'L' or 'R'."],
If[Length[Flatten[Position[tangle[[1]],boundaryregion]]]!=1,Print["Input Error: The specified region is not a boundary region of the specified tangle."]
,
INS1=Flatten[Position[tangle[[1]],boundaryregion]][[1]];
INS2=If[INS1<Length[tangle[[1]]],INS1+1,1];
colour1=StrandColour[INS1,tangle];
colour2=StrandColour[INS2,tangle];
r2=StrandNeighbouringRegions[INS2,tangle][[2]];
r3=NextLetter[tangle];
r4=StrandNeighbouringRegions[INS1,tangle][[1]];
newrow=\!\(\*
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"{", 
RowBox[{"r1", ",", "r2", ",", "r3", ",", "r4", ",", "twist", ",", "colour2", ",", "colour1"}], "}"}], 
RowBox[{
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS1", ",", "tangle"}], "]"}], "==", "\"\<out\>\""}], "&&", 
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS2", ",", "tangle"}], "]"}], "==", "\"\<out\>\""}]}]},
{
RowBox[{"{", 
RowBox[{"r2", ",", "r3", ",", "r4", ",", "r1", ",", "twist", ",", "colour1", ",", "colour2"}], "}"}], 
RowBox[{
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS1", ",", "tangle"}], "]"}], "==", "\"\<out\>\""}], "&&", 
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS2", ",", "tangle"}], "]"}], "==", "\"\<in\>\""}]}]},
{
RowBox[{"{", 
RowBox[{"r3", ",", "r4", ",", "r1", ",", "r2", ",", "twist", ",", "colour2", ",", "colour1"}], "}"}], 
RowBox[{
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS1", ",", "tangle"}], "]"}], "==", "\"\<in\>\""}], "&&", 
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS2", ",", "tangle"}], "]"}], "==", "\"\<in\>\""}]}]},
{
RowBox[{"{", 
RowBox[{"r4", ",", "r1", ",", "r2", ",", "r3", ",", "twist", ",", "colour1", ",", "colour2"}], "}"}], 
RowBox[{
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS1", ",", "tangle"}], "]"}], "==", "\"\<in\>\""}], "&&", 
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS2", ",", "tangle"}], "]"}], "==", "\"\<out\>\""}]}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False], 
RowBox[{"twist", "===", "L"}]},
{
TagBox[GridBox[{
{"\[Piecewise]", GridBox[{
{
RowBox[{"{", 
RowBox[{"r1", ",", "r2", ",", "r3", ",", "r4", ",", "twist", ",", "colour1", ",", "colour2"}], "}"}], 
RowBox[{
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS1", ",", "tangle"}], "]"}], "==", "\"\<out\>\""}], "&&", 
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS2", ",", "tangle"}], "]"}], "==", "\"\<out\>\""}]}]},
{
RowBox[{"{", 
RowBox[{"r2", ",", "r3", ",", "r4", ",", "r1", ",", "twist", ",", "colour2", ",", "colour1"}], "}"}], 
RowBox[{
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS1", ",", "tangle"}], "]"}], "==", "\"\<out\>\""}], "&&", 
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS2", ",", "tangle"}], "]"}], "==", "\"\<in\>\""}]}]},
{
RowBox[{"{", 
RowBox[{"r3", ",", "r4", ",", "r1", ",", "r2", ",", "twist", ",", "colour1", ",", "colour2"}], "}"}], 
RowBox[{
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS1", ",", "tangle"}], "]"}], "==", "\"\<in\>\""}], "&&", 
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS2", ",", "tangle"}], "]"}], "==", "\"\<in\>\""}]}]},
{
RowBox[{"{", 
RowBox[{"r4", ",", "r1", ",", "r2", ",", "r3", ",", "twist", ",", "colour2", ",", "colour1"}], "}"}], 
RowBox[{
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS1", ",", "tangle"}], "]"}], "==", "\"\<in\>\""}], "&&", 
RowBox[{
RowBox[{"StrandOrient", "[", 
RowBox[{"INS2", ",", "tangle"}], "]"}], "==", "\"\<out\>\""}]}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False], 
RowBox[{"twist", "===", "R"}]}
},
AllowedDimensions->{2, Automatic},
Editable->True,
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.84]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
Selectable->True]}
},
GridBoxAlignment->{"Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxItemSize->{"Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}},
GridBoxSpacings->{"Columns" -> {Offset[0.27999999999999997`], {Offset[0.35]}, Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {Offset[0.2], {Offset[0.4]}, Offset[0.2]}, "RowsIndexed" -> {}, "Items" -> {}, "ItemsIndexed" -> {}}],
"Piecewise",
DeleteWithContents->True,
Editable->False,
SelectWithContents->True,
Selectable->False]\);
Join[{tangle[[1]]},
Join[Drop[tangle,1][[All,1;;4]]/.{boundaryregion->r3},Drop[tangle,1][[All,5;;7]],2],
{newrow}
]
]
]
];
(* closing a tangle *)
Closure::usage="Closure[tangle_,boundaryregion_] computes the Alexander polynomial of the closure of the tangle 'tangle' at the region 'boundaryregion'.";
Closure[tangle_,boundaryregion_]:=
Module[
{
db=tangle[[1]],
boundarypairs,
relevantsites,
poly,
EndPairs,
dbb=Range[Length[tangle[[1]]]]
}
,
If[MemberQ[db,boundaryregion]==False,
Print["Input Error: The specified region 'boundaryregion' is not a boundary region of the tangle 'tangle'."]
,
EndPairs=Transpose[{Reverse[RotateLeft[dbb,#]],RotateLeft[dbb,#]}[[All,1;;Length[dbb]/2]]&@@Position[db,boundaryregion][[1]]];
If[Length[Intersection[Map[StrandOrient[#,tangle]&,EndPairs,{2}],{{"out","out"},{"in","in"}}]]>0,
Print["Input Error: We cannot close at the specified region, because the strand orientations do not match up!"]
,
If[Length[Select[Map[StrandColour[#,tangle]&,EndPairs,{2}],#[[1]]=!=#[[2]]&]]>0,
Print["Input Error: We cannot close at the specified region, because the strand colours do not match up!"]
,
If[
OddQ[Length[tangle[[1]]]]
,
Print["There is something wrong with the input data of the tangle; there are an odd number of boundary regions!"]
,
If[
Length[tangle[[1]]]<3,
Print["This is a 2-ended tangle, you cannot close it!"],
boundarypairs=Transpose[{Reverse[RotateLeft[db,#-1]],RotateLeft[db,#]}[[All,1;;Length[db]/2-2]]&@@Position[db,boundaryregion][[1]]];
relevantsites=DeleteCases[Barcs[db],v_/;MemberQ[v,boundaryregion]==False||AnyTrue[Complement[db,Join[Flatten[boundarypairs],{boundaryregion}]],MemberQ[v,#]&]||AnyTrue[boundarypairs,Length[Intersection[v,#]]!=1&]];
poly=AlexpolyP[tangle];
Total[Coefficient[poly,Times@@#]&/@relevantsites]
]
]
]
]
]
];
(* dot representation of generators with \[Delta]-grading for 4-ended tangles *)
dotsize=30;
(* a single dot *)
LDot[dotcolour_,deltagrading_]:=Graphics[{dotcolour,Disk[{0,0},0.1],Black,Text[If[OddQ[deltagrading],ToString[deltagrading]<>"/2",ToString[deltagrading/2]],{0,dotsize/70}],Transparent,Disk[{0,0},0.2]},ImageSize->dotsize];
(* List of dots for a single site (dot colour) from (Laurent) polynomial in Z[\[Delta]^\[PlusMinus]1] *)
LDots[dotcolour_,poly_]:=Flatten[MapIndexed[ConstantArray[LDot[dotcolour,(#2[[1]]-1)-2Deltaexp[poly]],#1]&,CoefficientList[Simplify[((poly+zeroz)*\[Delta]^Deltaexp[poly])/.{\[Delta]->dd^2},dd>0],dd]/.zeroz->0]];
(* List of dots for a single site (dot colour) from element of the module generated by {a,b,c,d} over the base ring Z[p,q,\[Delta]^\[PlusMinus]1] *)
AllLDots[poly_]:=If[poly===0," ",GraphicsRow[#,ImageSize->Length[#]*dotsize]]&@Join[
LDots[Red,poly/.{a->1,b->0,c->0,d->0}],
LDots[Blue,poly/.{a->0,b->1,c->0,d->0}],
LDots[Green,poly/.{a->0,b->0,c->1,d->0}],
LDots[Yellow,poly/.{a->0,b->0,c->0,d->1}]];
(* grid with dots labelled by \[Delta]-grading, input should be output of AlexpolyPd for 4-ended tangles *)
SimpleGridLDots[poly_]:=If[poly===0,Grid[{{0}},Frame->All,FrameStyle->Directive[Thickness[0.1],AbsoluteDashing[{100,0.01}],Gray], Background->{Gray}],Grid[Map[AllLDots,CoefficientList[(poly+zeroz)*p^Pexp[poly]*q^Qexp[poly],{p,q}]/.zeroz->0,{2}],Frame->All,FrameStyle->Directive[Thickness[0.1],AbsoluteDashing[{100,0.01}],Gray],Background->{{Qexp[poly]+1->LightGray},{Pexp[poly]+1->LightGray},{Pexp[poly]+1,Qexp[poly]+1}->Gray }]];
(* quick way to get dot diagram from tangle input data *)
DotGrid[tangle_]:=SimpleGridLDots[AlexpolyPd[tangle]];

EndPackage[]
